﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Threading;
using System.Windows.Threading;

namespace AsynchronousExamples
{
    [Export(typeof(IMeals))]
    public class MealProcessor : IMeals
    {
        private readonly DispatcherTimer _timer =
            new DispatcherTimer
                {
                    Interval = TimeSpan.FromSeconds(2)
                };

        private class AsyncRequest
        {
            public IAsyncResult Result;
            public AsyncCallback Callback;            
        }

        private class AsyncResult : IAsyncResult
        {
            public AsyncResult()
            {
                AsyncWaitHandle = null;
                CompletedSynchronously = false;
                IsCompleted = false;
            }
            public bool IsCompleted { get; private set; }

            public WaitHandle AsyncWaitHandle { get; private set; }
            
            public object AsyncState { get; set; }

            public bool CompletedSynchronously { get; private set; }            
        }

        private readonly Queue<AsyncRequest> _queuedRequests = 
            new Queue<AsyncRequest>();

        public MealProcessor()
        {
            _timer.Interval = TimeSpan.FromSeconds(1);
            _timer.Tick += TimerTick;
            _timer.Start();
        }

        private void TimerTick(object sender, EventArgs e)
        {
            if (_queuedRequests.Count <= 0) return;
            Monitor.Enter(((ICollection) _queuedRequests).SyncRoot);            
            var request = _queuedRequests.Dequeue();
            Monitor.Exit(((ICollection) _queuedRequests).SyncRoot);            
            request.Callback(request.Result);            
        }

        private IAsyncResult MakeAndQueueResult(
            AsyncCallback callback, 
            object userState)
        {
            var result = new AsyncResult
                             {
                                 AsyncState = userState
                             };
            var request = new AsyncRequest
                              {
                                  Callback = callback, 
                                  Result = result
                              };
            lock (((ICollection)_queuedRequests).SyncRoot)
            {
                _queuedRequests.Enqueue(request);
            }

            return result;
        }

        public IAsyncResult BeginBreakfast(
            AsyncCallback callback, 
            object userState)
        {
            return MakeAndQueueResult(callback, userState);
        }

        public bool EndBreakfast(IAsyncResult ar)
        {
            return true;
        }

        public IAsyncResult BeginLunch(
            AsyncCallback callback, 
            object userState)
        {
            return MakeAndQueueResult(callback, userState);
        }

        public bool EndLunch(IAsyncResult ar)
        {
            return true;
        }

        public IAsyncResult BeginDinner(
            AsyncCallback callback, 
            object userState)
        {
            return MakeAndQueueResult(callback, userState);
        }

        public bool EndDinner(IAsyncResult ar)
        {
            return true;
        }
    }
}