﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Linq;
using System.Reactive;
using System.Reactive.Linq;
using Jounce.Core.Command;
using Jounce.Core.ViewModel;
using Jounce.Core.Workflow;
using Jounce.Framework.Command;
using Jounce.Framework.Workflow;

namespace AsynchronousExamples
{
    [ExportAsViewModel(typeof (ViewModel))]
    public class ViewModel : BaseViewModel
    {
        [Import]
        public IMeals Meals { get; set; }

        [Import]
        public MealProxy MealProxy { get; set; }

        public ViewModel()
        {
            EatAsyncCommand = new ActionCommand<object>(
                obj => EatAsync(), obj => !_eating);
            EatEventCommand = new ActionCommand<object>(
                obj => EatEvent(), obj => !_eating);
            EatLambdaCommand = new ActionCommand<object>(
                obj => EatLambda(), obj => !_eating);
            EatWorkflowCommand = new ActionCommand<object>(
                obj => EatWorkflow(), obj => !_eating);
            EatRxCommand = new ActionCommand<object>(
                obj => EatRx(), obj => !_eating);
        }

        private bool _eating;

        private bool _breakfastAsync;

        public bool BreakfastAsync
        {
            get { return _breakfastAsync; }
            set
            {
                _breakfastAsync = value;
                RaisePropertyChanged(() => BreakfastAsync);
            }
        }

        private bool _lunchAsync;

        public bool LunchAsync
        {
            get { return _lunchAsync; }
            set
            {
                _lunchAsync = value;
                RaisePropertyChanged(() => LunchAsync);
            }
        }

        private bool _dinnerAsync;

        public bool DinnerAsync
        {
            get { return _dinnerAsync; }
            set
            {
                _dinnerAsync = value;
                RaisePropertyChanged(() => DinnerAsync);
            }
        }

        public IActionCommand EatAsyncCommand { get; private set; }
        public IActionCommand EatEventCommand { get; private set; }
        public IActionCommand EatLambdaCommand { get; private set; }
        public IActionCommand EatWorkflowCommand { get; private set; }
        public IActionCommand EatRxCommand { get; private set; }

        #region Asynchronous Programming Model 

        private void EatAsync()
        {
            StartEating();
            Meals.BeginBreakfast(EndBreakfast, null);
        }

        private void EndBreakfast(IAsyncResult ar)
        {
            BreakfastAsync = Meals.EndBreakfast(ar);
            Meals.BeginLunch(EndLunch, null);
        }

        private void EndLunch(IAsyncResult ar)
        {
            LunchAsync = Meals.EndBreakfast(ar);
            Meals.BeginDinner(
                arFinal =>
                    {
                        DinnerAsync = Meals.EndDinner(arFinal);
                        DoneEating();
                    }, null);
        }

        #endregion

        #region Event Model 

        private void EatEvent()
        {
            StartEating();
            MealProxy.BreakfastCompleted += MealProxyBreakfastCompleted;
            MealProxy.BreakfastAsync(null);
        }

        private void MealProxyBreakfastCompleted(object sender, MealProxy.MealCompletedEventArgs e)
        {
            BreakfastAsync = e.Result;
            MealProxy.BreakfastCompleted -= MealProxyBreakfastCompleted;
            MealProxy.LunchCompleted += MealProxyLunchCompleted;
            MealProxy.LunchAsync(null);
        }

        private void MealProxyLunchCompleted(object sender, MealProxy.MealCompletedEventArgs e)
        {
            LunchAsync = e.Result;
            MealProxy.LunchCompleted -= MealProxyLunchCompleted;
            MealProxy.DinnerCompleted += MealProxyDinnerCompleted;
            MealProxy.DinnerAsync(null);
        }

        private void MealProxyDinnerCompleted(object sender, MealProxy.MealCompletedEventArgs e)
        {
            DinnerAsync = e.Result;
            MealProxy.DinnerCompleted -= MealProxyDinnerCompleted;
            DoneEating();
        }

        #endregion

        #region Lambda expression 

        private void EatLambda()
        {
            StartEating();
            MealProxy.BreakfastCompleted +=
                (o, e) =>
                    {
                        Debug.WriteLine("Breakfast completed");
                        BreakfastAsync = e.Result;
                        MealProxy.LunchCompleted +=
                            (o1, e1) =>
                                {
                                    Debug.WriteLine("Lunch completed");
                                    LunchAsync = e.Result;
                                    MealProxy.DinnerCompleted +=
                                        (o2, e2) =>
                                            {
                                                Debug.WriteLine("Dinner Completed");
                                                DinnerAsync = e.Result;
                                                DoneEating();
                                            };
                                    MealProxy.DinnerAsync(null);
                                };
                        MealProxy.LunchAsync(null);
                    };
            MealProxy.BreakfastAsync(null);
        }

        #endregion

        #region Reactive 

        private IObservable<EventPattern<MealProxy.MealCompletedEventArgs>>
            _breakfastRx, _lunchRx, _dinnerRx;

        private void InitializeRx()
        {
            _breakfastRx = Observable
                .FromEventPattern<EventHandler<MealProxy.MealCompletedEventArgs>,
                    MealProxy.MealCompletedEventArgs>(
                        h => MealProxy.BreakfastCompleted += h,
                        h => MealProxy.BreakfastCompleted -= h)
                .Take(1);
            _lunchRx = Observable
                .FromEventPattern<EventHandler<MealProxy.MealCompletedEventArgs>,
                    MealProxy.MealCompletedEventArgs>(
                        h => MealProxy.LunchCompleted += h,
                        h => MealProxy.LunchCompleted -= h)
                .Take(1);
            _dinnerRx = Observable
                .FromEventPattern<EventHandler<MealProxy.MealCompletedEventArgs>,
                    MealProxy.MealCompletedEventArgs>(
                        h => MealProxy.DinnerCompleted += h,
                        h => MealProxy.DinnerCompleted -= h)
                .Take(1);
        }

        public void EatRx()
        {
            StartEating();
            InitializeRx();
            _breakfastRx.Subscribe(
                e =>
                    {
                        BreakfastAsync = e.EventArgs.Result;
                        MealProxy.LunchAsync(null);
                    });
            _lunchRx.Subscribe(
                e =>
                    {
                        LunchAsync = e.EventArgs.Result;
                        MealProxy.DinnerAsync(null);
                    });
            _dinnerRx.Subscribe(
                e =>
                    {
                        DinnerAsync = e.EventArgs.Result;
                        DoneEating();
                    });
            MealProxy.BreakfastAsync(null);
        }

        #endregion

        #region Workflow

        private void EatWorkflow()
        {
            WorkflowController.Begin(MealWorkflow());
        }

        private WorkflowEvent<MealProxy.MealCompletedEventArgs>
            _breakfast, _lunch, _dinner;

        private void InitializeWorkflow()
        {
            _breakfast = new WorkflowEvent<MealProxy.MealCompletedEventArgs>
                (() => MealProxy.BreakfastAsync(null),
                    h => MealProxy.BreakfastCompleted += h,
                    h => MealProxy.BreakfastCompleted -= h
                );
            _lunch = new WorkflowEvent<MealProxy.MealCompletedEventArgs>
                (() => MealProxy.LunchAsync(null),
                 h => MealProxy.LunchCompleted += h,
                 h => MealProxy.LunchCompleted -= h
                );
            _dinner = new WorkflowEvent<MealProxy.MealCompletedEventArgs>
                (() => MealProxy.DinnerAsync(null),
                 h => MealProxy.DinnerCompleted += h,
                 h => MealProxy.DinnerCompleted -= h
                );
        }

        private IEnumerable<IWorkflow> MealWorkflow()
        {
            InitializeWorkflow();
            StartEating();
            yield return _breakfast;
            BreakfastAsync = _breakfast.Result.Result;
            yield return _lunch;
            LunchAsync = _lunch.Result.Result;
            yield return _dinner;
            DinnerAsync = _dinner.Result.Result;
            DoneEating();
        }

        #endregion

        private void StartEating()
        {
            _eating = true;
            BreakfastAsync = false;
            LunchAsync = false;
            DinnerAsync = false;
            ExecuteChanged();
        }

        private void DoneEating()
        {
            _eating = false;
            ExecuteChanged();
        }

        private void ExecuteChanged()
        {
            EatAsyncCommand.RaiseCanExecuteChanged();
            EatEventCommand.RaiseCanExecuteChanged();
            EatLambdaCommand.RaiseCanExecuteChanged();
            EatWorkflowCommand.RaiseCanExecuteChanged();
            EatRxCommand.RaiseCanExecuteChanged();            
        }
    }
}