using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;

namespace DynamicDataExample
{
    /// <summary>
    ///     Helper for dynamic, bindable types
    /// </summary>
    /// <typeparam name="T">The type</typeparam>
    public class CustomTypeHelper<T> : ICustomTypeProvider, INotifyPropertyChanged
    {
        private static readonly List<CustomPropertyInfoHelper> 
            CustomProperties = new List<CustomPropertyInfoHelper>();
        private readonly Dictionary<string, object> _customPropertyValues;
        private CustomTypeDelegate _ctype;
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }

        public CustomTypeHelper()
        {
            _customPropertyValues = new Dictionary<string, object>();
            foreach (var property in GetCustomType().GetProperties())
            {
                _customPropertyValues.Add(property.Name, null);
            }
        }

        public static void AddProperty(string name)
        {
            if (!CheckIfNameExists(name))
                CustomProperties.Add(new CustomPropertyInfoHelper(name, typeof (String)));
        }

        public static void AddProperty(string name, Type propertyType)
        {
            if (!CheckIfNameExists(name))
                CustomProperties.Add(new CustomPropertyInfoHelper(name, propertyType));
        }

        public static void AddProperty(string name, Type propertyType, List<Attribute> attributes)
        {
            if (!CheckIfNameExists(name))
                CustomProperties.Add(new CustomPropertyInfoHelper(name, propertyType, attributes));
        }

        private static bool CheckIfNameExists(string name)
        {
            if ((from p in CustomProperties select p.Name).Contains(name) ||
                (from p in typeof (T).GetProperties() select p.Name).Contains(name))
                throw new Exception("The property with this name already exists: " + name);
            return false;
        }

        public void SetPropertyValue(string propertyName, object value)
        {
            var propertyInfo =
                (from prop in CustomProperties where prop.Name == propertyName select prop).FirstOrDefault();
            if (!_customPropertyValues.ContainsKey(propertyName))
                throw new Exception("There is no property " + propertyName);
            if (propertyInfo != null && !ValidateValueType(value, propertyInfo.Type))
                throw new Exception("Value is of the wrong type or null for a non-nullable type.");
            if (_customPropertyValues[propertyName] == value) return;

            _customPropertyValues[propertyName] = value;
            NotifyPropertyChanged(propertyName);
        }

        private static bool ValidateValueType(object value, Type type)
        {
            if (value == null)
                // Non-value types can be assigned null.
                if (!type.IsValueType)
                    return true;
                else
                    // Check if the type if a Nullable type.
                    return (type.IsGenericType && type.GetGenericTypeDefinition() == typeof (Nullable<>));
            return type.IsAssignableFrom(value.GetType());
        }

        public object GetPropertyValue(string propertyName)
        {
            if (_customPropertyValues.ContainsKey(propertyName))
                return _customPropertyValues[propertyName];
            throw new Exception("There is no property " + propertyName);
        }

        public PropertyInfo[] GetProperties()
        {
            return GetCustomType().GetProperties();
        }

        public Type GetCustomType()
        {
            return _ctype ?? (_ctype = new CustomTypeDelegate(typeof (T)));
        }

        private class CustomTypeDelegate : Type
        {
            private readonly Type _baseType;

            public CustomTypeDelegate(Type delegatingType)
            {
                _baseType = delegatingType;
            }

            public override Assembly Assembly
            {
                get { return _baseType.Assembly; }
            }

            public override string AssemblyQualifiedName
            {
                get { return _baseType.AssemblyQualifiedName; }
            }

            public override Type BaseType
            {
                get { return _baseType.BaseType; }
            }

            public override string FullName
            {
                get { return _baseType.FullName; }
            }

            public override Guid GUID
            {
                get { return _baseType.GUID; }
            }

            protected override TypeAttributes GetAttributeFlagsImpl()
            {
                throw new NotImplementedException();
            }

            protected override ConstructorInfo GetConstructorImpl(BindingFlags bindingAttr, Binder binder,
                                                                  CallingConventions callConvention, Type[] types,
                                                                  ParameterModifier[] modifiers)
            {
                throw new NotImplementedException();
            }

            public override ConstructorInfo[] GetConstructors(BindingFlags bindingAttr)
            {
                return _baseType.GetConstructors(bindingAttr);
            }

            public override Type GetElementType()
            {
                return _baseType.GetElementType();
            }

            public override EventInfo GetEvent(string name, BindingFlags bindingAttr)
            {
                return _baseType.GetEvent(name, bindingAttr);
            }

            public override EventInfo[] GetEvents(BindingFlags bindingAttr)
            {
                return _baseType.GetEvents(bindingAttr);
            }

            public override FieldInfo GetField(string name, BindingFlags bindingAttr)
            {
                return _baseType.GetField(name, bindingAttr);
            }

            public override FieldInfo[] GetFields(BindingFlags bindingAttr)
            {
                return _baseType.GetFields(bindingAttr);
            }

            public override Type GetInterface(string name, bool ignoreCase)
            {
                return _baseType.GetInterface(name, ignoreCase);
            }

            public override Type[] GetInterfaces()
            {
                return _baseType.GetInterfaces();
            }

            public override MemberInfo[] GetMembers(BindingFlags bindingAttr)
            {
                return _baseType.GetMembers(bindingAttr);
            }

            protected override MethodInfo GetMethodImpl(string name, BindingFlags bindingAttr, Binder binder,
                                                        CallingConventions callConvention, Type[] types,
                                                        ParameterModifier[] modifiers)
            {
                throw new NotImplementedException();
            }

            public override MethodInfo[] GetMethods(BindingFlags bindingAttr)
            {
                return _baseType.GetMethods(bindingAttr);
            }

            public override Type GetNestedType(string name, BindingFlags bindingAttr)
            {
                return _baseType.GetNestedType(name, bindingAttr);
            }

            public override Type[] GetNestedTypes(BindingFlags bindingAttr)
            {
                return _baseType.GetNestedTypes(bindingAttr);
            }

            public override PropertyInfo[] GetProperties(BindingFlags bindingAttr)
            {
                var clrProperties = _baseType.GetProperties(bindingAttr);
                return clrProperties != null
                           ? clrProperties.Concat(CustomProperties).ToArray()
                           : CustomProperties.ToArray();
            }

            protected override PropertyInfo GetPropertyImpl(string name, BindingFlags bindingAttr, Binder binder,
                                                            Type returnType, Type[] types, ParameterModifier[] modifiers)
            {
                // Look for the CLR property with this name first.
                var propertyInfo =
                    (from prop in GetProperties(bindingAttr) where prop.Name == name select prop).FirstOrDefault() ??
                    (from prop in CustomProperties where prop.Name == name select prop).FirstOrDefault();
                return propertyInfo;
            }

            protected override bool HasElementTypeImpl()
            {
                throw new NotImplementedException();
            }

            public override object InvokeMember(string name, BindingFlags invokeAttr, Binder binder, object target,
                                                object[] args, ParameterModifier[] modifiers,
                                                System.Globalization.CultureInfo culture, string[] namedParameters)
            {
                return _baseType.InvokeMember(name, invokeAttr, binder, target, args, modifiers, culture,
                                              namedParameters);
            }

            protected override bool IsArrayImpl()
            {
                throw new NotImplementedException();
            }

            protected override bool IsByRefImpl()
            {
                throw new NotImplementedException();
            }

            protected override bool IsCOMObjectImpl()
            {
                throw new NotImplementedException();
            }

            protected override bool IsPointerImpl()
            {
                throw new NotImplementedException();
            }

            protected override bool IsPrimitiveImpl()
            {
                return _baseType.IsPrimitive;
            }

            public override Module Module
            {
                get { return _baseType.Module; }
            }

            public override string Namespace
            {
                get { return _baseType.Namespace; }
            }

            public override Type UnderlyingSystemType
            {
                get { return _baseType.UnderlyingSystemType; }
            }

            public override object[] GetCustomAttributes(Type attributeType, bool inherit)
            {
                return _baseType.GetCustomAttributes(attributeType, inherit);
            }

            public override object[] GetCustomAttributes(bool inherit)
            {
                return _baseType.GetCustomAttributes(inherit);
            }

            public override bool IsDefined(Type attributeType, bool inherit)
            {
                return _baseType.IsDefined(attributeType, inherit);
            }

            public override string Name
            {
                get { return _baseType.Name; }
            }
        }

        // Custom implementation of the PropertyInfo
        private class CustomPropertyInfoHelper : PropertyInfo
        {
            private readonly string _name;
            public readonly Type Type;
            private readonly List<Attribute> _attributes = new List<Attribute>();

            public CustomPropertyInfoHelper(string name, Type type)
            {
                _name = name;
                Type = type;
            }

            public CustomPropertyInfoHelper(string name, Type type, List<Attribute> attributes)
            {
                _name = name;
                Type = type;
                _attributes = attributes;
            }

            public override PropertyAttributes Attributes
            {
                get { throw new NotImplementedException(); }
            }

            public override bool CanRead
            {
                get { return true; }
            }

            public override bool CanWrite
            {
                get { return true; }
            }

            public override MethodInfo[] GetAccessors(bool nonPublic)
            {
                throw new NotImplementedException();
            }

            public override MethodInfo GetGetMethod(bool nonPublic)
            {
                throw new NotImplementedException();
            }

            public override ParameterInfo[] GetIndexParameters()
            {
                throw new NotImplementedException();
            }

            public override MethodInfo GetSetMethod(bool nonPublic)
            {
                throw new NotImplementedException();
            }

            // Returns the value from the dictionary stored in the Customer's instance.
            public override object GetValue(object obj, BindingFlags invokeAttr, Binder binder, object[] index,
                                            System.Globalization.CultureInfo culture)
            {
                return obj.GetType().GetMethod("GetPropertyValue").Invoke(obj, new[] {_name});
            }

            public override Type PropertyType
            {
                get { return Type; }
            }

            // Sets the value in the dictionary stored in the Customer's instance.
            public override void SetValue(object obj, object value, BindingFlags invokeAttr, Binder binder,
                                          object[] index, System.Globalization.CultureInfo culture)
            {
                obj.GetType().GetMethod("SetPropertyValue").Invoke(obj, new[] {_name, value});
            }

            public override Type DeclaringType
            {
                get { throw new NotImplementedException(); }
            }

            public override object[] GetCustomAttributes(Type attributeType, bool inherit)
            {
                var attrs = from a in _attributes where a.GetType() == attributeType select a;
                return attrs.ToArray();
            }

            public override object[] GetCustomAttributes(bool inherit)
            {
                return _attributes.ToArray();
            }

            public override bool IsDefined(Type attributeType, bool inherit)
            {
                throw new NotImplementedException();
            }

            public override string Name
            {
                get { return _name; }
            }

            public override Type ReflectedType
            {
                get { throw new NotImplementedException(); }
            }
        }
    }
}