﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace FluentValidation
{
    public class BaseEntity 
        : INotifyPropertyChanged, INotifyDataErrorInfo
    {
        private readonly Dictionary<string, List<string>> _errors 
            = new Dictionary<string, List<string>>();               

        public void ClearErrors(string propertyName)
        {
            if (_errors.ContainsKey(propertyName))
            {
                _errors[propertyName].Clear();
            }

            RaiseErrorChange(propertyName);
        }

        public void SetError(string propertyName, string message)
        {
            if (!_errors.ContainsKey(propertyName))
            {
                _errors.Add(
                    propertyName, 
                    new List<string>());
            }

            _errors[propertyName].Add(message);
            RaiseErrorChange(propertyName);
        }

        public IEnumerable GetErrors(string propertyName)
        {
            if (_errors.ContainsKey(propertyName) &&
                _errors[propertyName].Count > 0)
            {
                return _errors[propertyName];
            }            

            return null;
        }

        public bool HasErrors
        {
            get
            {
                return _errors.Keys.Any(
                    key => _errors[key].Count > 0);
            }
        }
        
        protected void RaisePropertyChange(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(
                    this, 
                    new PropertyChangedEventArgs(propertyName));
            }
        }

        protected void RaiseErrorChange(string propertyName)
        {
            var handler = ErrorsChanged;
            if (handler != null)
            {
                handler(
                    this, 
                    new DataErrorsChangedEventArgs(propertyName));
            }
        }
        
        public event PropertyChangedEventHandler PropertyChanged;
        
        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;
    }
}