﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using IsolatedStorageExample.Encryption;
using IsolatedStorageExample.Model;
using Jounce.Core.ViewModel;
using Jounce.Framework;
using Jounce.Framework.Command;

namespace IsolatedStorageExample.ViewModel
{
    [ExportAsViewModel(typeof (NotesViewModel))]
    public class NotesViewModel : BaseViewModel
    {
        private readonly Signing _signing = new Signing();
        private readonly Encrypting _encrypting = new Encrypting();

        public NotesViewModel()
        {
            Categories = new ObservableCollection<Category>();
            IncreaseQuotaCommand = new ActionCommand<object>(
                obj => IncreaseQuota());
            AddCategoryCommand = new ActionCommand<object>(
                obj => AddCategory());
            AddNoteCommand = new ActionCommand<object>(
                obj => AddNote());
            SaveNoteCommand = new ActionCommand<object>(
                obj => SaveNote());

        }

        private void SaveNote()
        {
            if (CurrentNote != null)
            {
                try
                {
                    UpdateNote(CurrentCategory, CurrentNote);
                }
                catch(Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void AddNote()
        {
            if (string.IsNullOrEmpty(_newNote))
            {
                MessageBox.Show("Must enter title for note.");
                return;
            }

            if (CurrentCategory == null)
            {
                MessageBox.Show("You must select a category to add a new note.");
                return;
            }

            try
            {
                var note = new Note {Name = _newNote, Contents = string.Empty};
                UpdateNote(CurrentCategory, note);
                CurrentNote = note;
                Refresh();
                NewNote = string.Empty;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void AddCategory()
        {
            if (string.IsNullOrEmpty(_newCategory))
            {
                MessageBox.Show("Must enter category.");
                return;
            }

            try
            {
                var category = new Category {Name = _newCategory};
                CreateCategory(category);
                CurrentCategory = category;
                Refresh();
                NewCategory = string.Empty;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }            
        }

        private void Refresh()
        {
            var category = CurrentCategory;
            var note = CurrentNote;
            Categories.Clear();
            LoadCategories();
            CurrentCategory = (from c
                                    in Categories
                                where c.Name.Equals(category.Name)
                                select c).FirstOrDefault();
            if (note != null && CurrentCategory != null)
            {
                CurrentNote = (from n in CurrentCategory.Notes
                                where n.Name.Equals(note.Name)
                                select n).FirstOrDefault();
            }
        }

        private void IncreaseQuota()
        {
            using (var iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                bool result;
                try
                {
                    result = iso.IncreaseQuotaTo(20*1024*1024);
                }
                catch
                {
                    result = false;
                }
                if (result)
                {
                    LoadCategories();
                    EventAggregator.Publish("MainControl".AsViewNavigationArgs());   
                }
                else
                {
                    EventAggregator.Publish("IsoWarning".AsViewNavigationArgs());
                }
            }
        }

        private static void CreateCategory(Category category)
        {
            using (var iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                iso.CreateDirectory(category.Name);                
            }
        }

        private void UpdateNote(Category category, Note note)
        {
            using (var iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (!iso.DirectoryExists(category.Name))
                {
                    CreateCategory(category);
                }
                var path = Path.Combine(
                    category.Name, string.Format("{0}{1}", note.Name, ".txt"));
                var stream
                    = new StreamWriter(iso.OpenFile(path, FileMode.Create, FileAccess.Write));
                _encrypting.Encrypt(note, stream.BaseStream);
                var signatureFolder = Path.Combine(category.Name, "Signatures");
                if (!iso.DirectoryExists(signatureFolder))
                {
                    iso.CreateDirectory(signatureFolder);
                }
                var signaturePath = Path.Combine(
                signatureFolder, string.Format("{0}{1}", note.Name, ".signature"));
                _signing.CreateSignature(path, signaturePath);
            }            
        }

        private void LoadCategories()
        {
            using (var iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                foreach (var fileName in iso.GetDirectoryNames())
                {
                    var category = new Category
                                        {
                                            Name = fileName
                                        };
                    foreach (var noteName in iso.GetFileNames(string.Format(@"{0}\*",
                        fileName)))
                    {
                        var path = Path.Combine(category.Name, noteName);
                        Note note;
                        using (var stream = new StreamReader(
                            iso.OpenFile(path, FileMode.Open, FileAccess.Read)))                          
                        {
                            note = new Note
                                            {
                                                Name = noteName.Substring(
                                                0,
                                                noteName.Length - 4)                                                
                                            };
                            _encrypting.Decrypt(note, stream.BaseStream);
                            category.Notes.Add(note);                            
                        }
                        var signatureFolder = Path.Combine(category.Name, "Signatures");
                        var signaturePath = Path.Combine(
                        signatureFolder, string.Format("{0}{1}", note.Name, ".signature"));
                        _signing.VerifySignature(path, signaturePath);
                    }
                    Categories.Add(category);
                }
            }
        }

        private string _newCategory;
        public string NewCategory
        {
            get { return _newCategory; }
            set
            {
                _newCategory = value;
                RaisePropertyChanged(() => NewCategory);                
            }
        }

        private string _newNote;
        public string NewNote
        {
            get { return _newNote; }
            set
            {
                _newNote = value;
                RaisePropertyChanged(() => NewNote);
            }
        }

        public ObservableCollection<Category> Categories { get; private set; }

        private Category _currentCategory;

        public Category CurrentCategory
        {
            get { return _currentCategory; }
            set
            {
                _currentCategory = value;
                RaisePropertyChanged(() => CurrentCategory);
            }
        }

        private Note _currentNote;

        public Note CurrentNote
        {
            get { return _currentNote; }
            set
            {
                _currentNote = value;
                RaisePropertyChanged(() => CurrentNote);
            }
        }

        public ICommand AddCategoryCommand { get; private set; }
        public ICommand AddNoteCommand { get; private set; }
        public ICommand SaveNoteCommand { get; private set; }
        public ICommand IncreaseQuotaCommand { get; private set; }

        protected override void InitializeVm()
        {
            try
            {
                var test = IsolatedStorageFile.GetUserStoreForApplication();
                test.Dispose();
            }
            catch
            {
                EventAggregator.Publish("IsoWarning".AsViewNavigationArgs());
                return;
            }

            using (var iso = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (iso.Quota < 20*1024*1024)
                {
                    EventAggregator.Publish("QuotaIncrease".AsViewNavigationArgs());
                    return;
                }

                if (iso.GetDirectoryNames().Length == 0)
                {
                    var cat = new Category {Name = "MyToDoList"};
                    var note = new Note
                                    {
                                        Name = "To Do", 
                                        Contents = "This is a do to item."
                                    };
                    UpdateNote(cat, note);
                }
            }

            LoadCategories();
            EventAggregator.Publish("MainControl".AsViewNavigationArgs());

            base.InitializeVm();
        }
    }
}