﻿using System;
using System.Collections;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Script.Serialization;
using ToDoList.Contracts;
using ToDoList.Model;

namespace ToDoList.Web
{
    public class RESTHandler : IHttpHandler
    {
        public const string REST_ERROR_TYPE = "todolist/error+json";
        public const string REST_TASK_TYPE = "todolist/task+json";
        public const string REST_TASKS_TYPE = "todolist/tasks+json";
        public const string REST_RESOURCES_TYPE = "todolist/resources+json";
                         
        public const string REST_PATH_ROOT = "/REST/";                        
        public const string REST_PATH_TASK = "/REST/Task/{0}";
        public const string REST_PATH_TASKS = "/REST/Tasks/";                   

        [Import]
        public IRepository Repository { get; set; }        

        #region IHttpHandler Members

        public bool IsReusable
        {
            // Return false in case your Managed Handler cannot be reused for another request.
            // Usually this would be false in case you have some state information preserved per request.
            get { return true; }
        }

        private static Uri MakeUri(HttpContext context, string relativePath)
        {
            return new Uri(
                string.Format("{0}://{1}:{2}{3}",
                                context.Request.Url.Scheme,
                                context.Request.Url.Host,
                                context.Request.Url.Port,
                                relativePath), UriKind.Absolute);
        }

        public class Error
        {
            public string ErrorText { get; set; }
        }

        public void ProcessRequest(HttpContext context)
        {
            try
            {
                if (Repository == null)
                {
                    MEFBootstrapper.SatisfyImports(this);
                }
                
                var paths = context.Request.Url.PathAndQuery.Split(
                    new[] {"/"}, StringSplitOptions.RemoveEmptyEntries);
                
                if (paths.Length <= 1)
                {
                    ShowResources(context);
                }
                else
                {
                    var method = paths[1];
                    var parm = paths.Length > 2 
                        ? paths[2] : string.Empty;
                    GetType().GetMethod(method).Invoke(
                        this,
                        new object[] {context, parm});
                }
            }
            catch (Exception ex)
            {
                context.Response.Clear();
                context.Response.ContentType = REST_ERROR_TYPE;
                WriteResponse(context, 
                    new Error
                        {
                            ErrorText = ex.Message
                        });
            }
        }
        
        public void Task(HttpContext context, string parm)
        {
            context.Response.ContentType = REST_TASK_TYPE;
            var key = Guid.Parse(parm);

            if (context.Request.HttpMethod.Equals("post", 
                StringComparison.InvariantCultureIgnoreCase))
            {
                var jsonBytes = new byte[context.Request.InputStream.Length];
                context.Request.InputStream.Read(jsonBytes, 0, jsonBytes.Length);
                var json = Encoding.UTF8.GetString(jsonBytes);
                {
                    var serializer = new JavaScriptSerializer();
                    var todo = serializer.Deserialize<ToDoItemOverride>(json);
                    Repository.Save(todo);
                }
            }

            var task = Repository.Query()
                .Where(t => t.Id.Equals(key))
                .FirstOrDefault();           
            WriteResponse(context, TaskResource(context, task));
        }        

        public void Tasks(HttpContext context, string parm)
        {
            context.Response.ContentType = REST_TASKS_TYPE;
            var list = Repository.Query()
                .Select(item => TaskResource(context, item))
                .ToList();
            var listResource = new
            {
                Type = typeof (ICollection).FullName,
                ListType = typeof (IToDoItem).FullName,
                Item = list,
                Location = MakeUri(context, REST_PATH_TASKS),
                Back = MakeUri(context, REST_PATH_ROOT)
            };
            WriteResponse(context, listResource);
        }

        private static object TaskResource(HttpContext context, IToDoItem item)
        {
            return new
            {
                Type = typeof(IToDoItem).FullName,
                Item = item,
                Location = MakeUri(context,
                string.Format(REST_PATH_TASK,
                                    item == null
                                        ? Guid.Empty
                                        : item.Id)),
                Back = MakeUri(context, REST_PATH_TASKS)
            };
        }

        private static void ShowResources(HttpContext context)
        {
            context.Response.ContentType = REST_RESOURCES_TYPE;
            var returnValue = new object[]
            {
                new
                    {
                        Resource = "Tasks",
                        Location = MakeUri(context, REST_PATH_TASKS),
                        Back = MakeUri(context, REST_PATH_ROOT)
                    }                      
            };
            WriteResponse(context, returnValue);
        }

        private static void WriteResponse(HttpContext context, object returnValue)
        {
            var sb = new StringBuilder();
            var jss = new JavaScriptSerializer();
            jss.Serialize(returnValue, sb);
            context.Response.Write(sb.ToString());
        }

        #endregion
    }
}