﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using Jounce.Core.Event;
using Jounce.Core.Workflow;
using Jounce.Framework.Workflow;
using ToDoList.Contracts;
using ToDoList.Model;
using ToDoList.Web;

namespace ToDoList.Repository
{
    [Export(typeof (IRepository))]
    public class WcfRiaRepository : IRepository, IPartImportsSatisfiedNotification, IEventSink<MessageToDoItemComplete>
    {
        private readonly ToDoRiaContext _context = new ToDoRiaContext();
        private EntitySet<ToDoItemRia> _tasks;

        private readonly Mapper<ToDoItemRia, ToDoItem> _mapper =
            new Mapper<ToDoItemRia, ToDoItem>(
                new[] {"Id", "CompletedDate", "IsComplete"});

        private Guid _concurrencyId = Guid.NewGuid();

        private bool _loadingTasks;

        [Import]
        public ExportFactory<ToDoItemOverride> ToDoItemFactory { get; set; }

        [Import]
        public IEventAggregator Messaging { get; set; }

        public WcfRiaRepository()
        {
            _tasks = _context.ToDoItemRias;
            LoadTasks();
            WorkflowController.Begin(ConcurrencyChecker());
        }

        private void LoadTasks()
        {
            _loadingTasks = true;

            var query = _context.GetTasksQuery();
            _context.Load(query, LoadBehavior.RefreshCurrent, LoadCallback, null);
        }

        private void LoadCallback(LoadOperation<ToDoItemRia> obj)
        {
            _tasks = _context.EntityContainer.GetEntitySet<ToDoItemRia>();
            RaiseAggregateRootChanged();
        }

        /// <summary>
        /// Iterates to check whether the task list has changed
        /// </summary>
        /// <returns>A never-ending workflow to poll the server for changes</returns>
        private IEnumerable<IWorkflow> ConcurrencyChecker()
        {
            IWorkflow delay = new WorkflowDelay(TimeSpan.FromSeconds(5));
            IWorkflow checkConcurrencyAction = null;
            Action checkConcurrency =
                () =>
                _context.GetConcurrencyId(
                    invokeOp =>
                        {
                            if (!invokeOp.HasError)
                            {
                                CheckConcurrencyAction(invokeOp.Value);
                            }
                            checkConcurrencyAction.Invoked();
                        }, null);
            checkConcurrencyAction = new WorkflowAction(checkConcurrency, false);

            while (!_concurrencyId.Equals(Guid.Empty))
            {
                if (!_loadingTasks)
                {
                    yield return checkConcurrencyAction;
                }
                yield return delay;
            }
        }

        private void CheckConcurrencyAction(Guid value)
        {
            if (value.Equals(_concurrencyId) || _loadingTasks) return;

            _concurrencyId = value;
            LoadTasks();
        }

        public IQueryable<IToDoItem> Query()
        {
            return _tasks.Select(FromRia).AsQueryable();
        }

        public void Save(IToDoItem item)
        {
            var existing = GetById(item.Id);
            if (existing != null)
            {
                FromTask(existing, item);
            }
            else
            {
                _tasks.Add(FromTask(item));
            }
            _context.SubmitChanges(
                s => RaiseAggregateRootChanged(),
                null);
        }

        public void Delete(IToDoItem item)
        {
            var riaItem = GetById(item.Id);
            if (riaItem == null) return;
            _context.ToDoItemRias.Remove(riaItem);
            _context.SubmitChanges(
                s => RaiseAggregateRootChanged(),
                null);
        }

        public Guid GetConcurrencyIdentifier
        {
            get { return _concurrencyId; }
        }

        private ToDoItemRia GetById(Guid id)
        {
            return (from t in _tasks
                    where t.Id.Equals(id)
                    select t).FirstOrDefault();
        }

        private IToDoItem FromRia(ToDoItemRia task)
        {
            var newTask = ToDoItemFactory.CreateExport().Value;
            _mapper.MapSourceToTarget(task, newTask);
            newTask.Id = task.Id;
            newTask.IsComplete = task.IsComplete;
            newTask.CompletedDate = task.CompletedDate;
            return newTask;
        }

        private void FromTask(ToDoItemRia riaItem, IToDoItem task)
        {
            _mapper.MapTargetToSource((ToDoItem) task, riaItem);
        }

        private ToDoItemRia FromTask(IToDoItem task)
        {
            var newTask = new ToDoItemRia {Id = Guid.NewGuid()};
            FromTask(newTask, task);
            return newTask;
        }

        protected void RaiseAggregateRootChanged()
        {
            var handler = AggregateRootChanged;
            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
            _loadingTasks = false;
        }

        public event EventHandler AggregateRootChanged;

        public void OnImportsSatisfied()
        {
            Messaging.Subscribe(this);
        }

        public void HandleEvent(MessageToDoItemComplete publishedEvent)
        {
            var todo = publishedEvent.Item;
            _context.MarkComplete(todo.Id);
        }
    }
}