﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Windows.Threading;

namespace ValidationExamples
{
    public class SampleClass : INotifyPropertyChanged, IDataErrorInfo, INotifyDataErrorInfo
    {
        private int _rangeValue;

        private readonly Dictionary<string, List<string>> _errors
            = new Dictionary<string, List<string>>();

        [Range(1, 5, ErrorMessage = "The value must be between 1 and 5.")]
        public int RangeValue
        {
            get { return _rangeValue; }
            set
            {
                Validator.ValidateProperty(value,
                                           new ValidationContext(this, null, null)
                                               {
                                                   MemberName = "RangeValue"
                                               });
                _rangeValue = value;
                RaisePropertyChanged("RangeValue");
            }
        }

        private int _exceptionValue;

        public string ExceptionValue
        {
            get { return _exceptionValue.ToString(); }
            set
            {
                int testValue;

                if (!int.TryParse(value, out testValue))
                {
                    throw new ValidationException("The value must be a valid integer.");
                }

                if (testValue < 1 || testValue > 5)
                {
                    throw new ValidationException("The value must be between 1 and 5.");
                }

                _exceptionValue = testValue;
                RaisePropertyChanged("ExceptionValue");
            }
        }

        private int _dataErrorValue;

        public string DataErrorValue
        {
            get { return _dataErrorValue.ToString(); }
            set
            {
                ClearErrors("DataErrorValue");

                int testValue;

                if (!int.TryParse(value, out testValue))
                {
                    SetError("DataErrorValue",
                             "The value must be a valid integer.");
                    return;
                }

                if (testValue < 1 || testValue > 5)
                {
                    SetError("DataErrorValue",
                             "The value must be between 1 and 5.");
                    return;
                }

                _dataErrorValue = testValue;
                RaisePropertyChanged("DataErrorValue");
            }
        }

        private int _notifyDataErrorValue;

        public string NotifyDataErrorValue
        {
            get { return _notifyDataErrorValue.ToString(); }
            set
            {
                ClearErrors("NotifyDataErrorValue");

                int testValue;

                if (!int.TryParse(value, out testValue))
                {
                    SetError("NotifyDataErrorValue",
                                "The value must be a valid integer.");
                    return;
                }

                ValidateField(
                    testValue, 
                    validated =>
                        {
                            if (!string.IsNullOrEmpty(validated))
                            {
                                SetError("NotifyDataErrorValue",
                                    validated);
                            }
                        }
                    );

                _notifyDataErrorValue = testValue;
                RaisePropertyChanged("NotifyDataErrorValue");
            }
        }

        private void ValidateField(int value, Action<string> validated)
        {
            var timer = new DispatcherTimer
                            {
                                Interval =
                                    TimeSpan.FromMilliseconds(100)
                            };
            timer.Tick +=
                (o, e) =>
                    {
                        timer.Stop();
                        validated(value >= 1 && value <= 5
                                      ? null
                                      : "The value must be between 1 and 5.");
                    };
            timer.Start();
        }

        protected void RaiseErrorChanged(string propertyName)
        {
            var handler = ErrorsChanged;
            if (handler != null)
            {
                handler(this, new DataErrorsChangedEventArgs(propertyName));
            }
        }

        protected void RaisePropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void ClearErrors(string propertyName)
        {            
            if (_errors.ContainsKey(propertyName))
            {
                _errors[propertyName].Clear();
            }

            RaiseErrorChanged(propertyName);
        }

        protected void SetError(string propertyName, string errorMessage)
        {
            if (!_errors.ContainsKey(propertyName))
            {
                _errors.Add(propertyName, new List<string>());
            }

            _errors[propertyName].Add(errorMessage);

            RaiseErrorChanged(propertyName);
        }

        public string this[string columnName]
        {
            get
            {
                if (!_errors.ContainsKey(columnName) ||
                    _errors[columnName].Count == 0)
                {
                    return null;
                }

                return string.Join(Environment.NewLine, _errors[columnName]);
            }
        }

        public string Error
        {
            get
            {
                return _errors.Keys.Any(key => _errors[key].Count > 0)
                           ? "Validation errors exist."
                           : null;
            }
        }

        public IEnumerable GetErrors(string propertyName)
        {
            var retVal = new List<string>();
            if (_errors.ContainsKey(propertyName))
            {
                retVal.AddRange(_errors[propertyName]);
            }
            return retVal; 
        }

        public bool HasErrors
        {
            get { return string.IsNullOrEmpty(Error); }
        }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;
    }
}